@@ -120,18 +120,24 @@ geary.mc <- function(x, listw, nsim, zero.policy=NULL,
                 var <- var[i]
                 return(geary(x=var, ...)$C)
             }
-            cl <- get("cl", envir = .spdepOptions)
-            if (!is.null(cl) && length(cl) > 1L) {
-                nnsim <- boot_wrapper_in(cl, nsim)
-                lres <- clusterCall(cl, boot, x, statistic=geary_boot,
-                    R=nnsim, sim="permutation", listw=listw, n=n, n1=wc$n1,
-                    S0=wc$S0, zero.policy=zero.policy)
-                res <- boot_wrapper_out(lres, match.call())
+            cores <- get.coresOption()
+            if (is.null(cores)) {
+            parallel <- "no"
             } else {
-                res <- boot(x, statistic=geary_boot, R=nsim,
-                    sim="permutation", listw=listw, n=n, n1=wc$n1, S0=wc$S0, 
-                    zero.policy=zero.policy)
+                parallel <- ifelse (get.mcOption(), "multicore", "snow")
             }
+            ncpus <- ifelse(is.null(cores), 1L, cores)
+            cl <- NULL
+            if (parallel == "snow") {
+                cl <- get.ClusterOption()
+                if (is.null(cl)) {
+                    parallel <- "no"
+                    warning("no cluster in ClusterOption, parallel set to no")
+                }
+            }
+            res <- boot(x, statistic=geary_boot, R=nsim,
+                sim="permutation", listw=listw, n=n, n1=wc$n1, S0=wc$S0, 
+                zero.policy=zero.policy, parallel=parallel, ncpus=ncpus, cl=cl)
             return(res)
         }
 	res <- numeric(length=nsim+1)
